@@ -147,10 +147,13 @@ def _multiply_seq_by_int(self, opnode, other, context):
 def _filter_uninferable_nodes(elts, context):
     for elt in elts:
         if elt is util.Uninferable:
-            yield elt
+            yield nodes.Unknown()
         else:
             for inferred in elt.infer(context):
-                yield inferred
+                if inferred is not util.Uninferable:
+                    yield inferred
+                else:
+                    yield nodes.Unknown()
 
 
 @decorators.yes_if_nothing_inferred
@@ -182,10 +185,11 @@ nodes.List.infer_binary_op = tl_infer_binary_op
 
 
 @decorators.yes_if_nothing_inferred
-def instance_infer_binary_op(self, opnode, operator, other, context, method):
+def instance_class_infer_binary_op(self, opnode, operator, other, context, method):
     return method.infer_call_result(self, context)
 
-bases.Instance.infer_binary_op = instance_infer_binary_op
+bases.Instance.infer_binary_op = instance_class_infer_binary_op
+nodes.ClassDef.infer_binary_op = instance_class_infer_binary_op
 
 
 # assignment ##################################################################
@@ -241,6 +245,10 @@ def _resolve_looppart(parts, asspath, context):
 
 @decorators.raise_if_nothing_inferred
 def for_assigned_stmts(self, node=None, context=None, asspath=None):
+    if isinstance(self, nodes.AsyncFor) or getattr(self, 'is_async', False):
+        # Skip inferring of async code for now
+        raise StopIteration(dict(node=self, unknown=node,
+                                 assign_path=asspath, context=context))
     if asspath is None:
         for lst in self.iter.infer(context):
             if isinstance(lst, (nodes.Tuple, nodes.List)):
@@ -355,7 +363,16 @@ def assign_assigned_stmts(self, node=None, context=None, asspath=None):
     raise StopIteration(dict(node=self, unknown=node,
                              assign_path=asspath, context=context))
 
+
+def assign_annassigned_stmts(self, node=None, context=None, asspath=None):
+    for inferred in assign_assigned_stmts(self, node, context, asspath):
+        if inferred is None:
+            yield util.Uninferable
+        else:
+            yield inferred
+
 nodes.Assign.assigned_stmts = assign_assigned_stmts
+nodes.AnnAssign.assigned_stmts = assign_annassigned_stmts
 nodes.AugAssign.assigned_stmts = assign_assigned_stmts
 
 
@@ -497,6 +514,11 @@ def with_assigned_stmts(self, node=None, context=None, asspath=None):
                         'Tried to infer a nonexistent target with index {index} '
                         'in {node!r}.', node=self, targets=node,
                         assign_path=asspath, context=context))
+                except TypeError:
+                    util.reraise(exceptions.InferenceError(
+                        'Tried to unpack an non-iterable value '
+                        'in {node!r}.', node=self, targets=node,
+                        assign_path=asspath, context=context))
             yield obj
     # Explicit StopIteration to return error information, see comment
     # in raise_if_nothing_inferred.
