@@ -14,6 +14,7 @@ new local scope in the language definition : Module, ClassDef, FunctionDef (and
 Lambda, GeneratorExp, DictComp and SetComp to some extent).
 """
 
+import sys
 import io
 import itertools
 import warnings
@@ -25,6 +26,7 @@ from astroid import context as contextmod
 from astroid import exceptions
 from astroid import decorators as decorators_mod
 from astroid.interpreter import objectmodel
+from astroid.interpreter import dunder_lookup
 from astroid import manager
 from astroid import mixins
 from astroid import node_classes
@@ -784,6 +786,8 @@ class FunctionDef(node_classes.Statement, Lambda):
         if isinstance(frame, ClassDef):
             if self.name == '__new__':
                 return 'classmethod'
+            elif sys.version_info >= (3, 6) and self.name == '__init_subclass__':
+                return 'classmethod'
             else:
                 type_name = 'method'
 
@@ -1095,6 +1099,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG,
                  col_offset=None, parent=None):
         self.instance_attrs = {}
         self.locals = {}
+        self.keywords = []
         self.bases = []
         self.body = []
         self.name = name
@@ -1104,7 +1109,8 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG,
             parent.frame().set_local(name, self)
 
     # pylint: disable=redefined-outer-name
-    def postinit(self, bases, body, decorators, newstyle=None, metaclass=None):
+    def postinit(self, bases, body, decorators, newstyle=None, metaclass=None, keywords=None):
+        self.keywords = keywords
         self.bases = bases
         self.body = body
         self.decorators = decorators
@@ -1509,6 +1515,34 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG,
                 pass
         return False
 
+    def getitem(self, index, context=None):
+        """Return the inference of a subscript.
+
+        This is basically looking up the method in the metaclass and calling it.
+        """
+        try:
+            methods = dunder_lookup.lookup(self, '__getitem__')
+        except exceptions.AttributeInferenceError as exc:
+            util.reraise(
+                exceptions.AstroidTypeError(
+                    node=self, error=exc,
+                    context=context
+                )
+            )
+
+        method = methods[0]
+
+        # Create a new callcontext for providing index as an argument.
+        if context:
+            new_context = context.clone()
+        else:
+            new_context = contextmod.InferenceContext()
+
+        new_context.callcontext = contextmod.CallContext(args=[index])
+        new_context.boundnode = self
+
+        return next(method.infer_call_result(self, new_context))
+
     def methods(self):
         """return an iterator on all methods defined in the class and
         its ancestors
@@ -1751,17 +1785,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG,
                 for base in baseobj.bases:
                     yield base
 
-    def mro(self, context=None):
-        """Get the method resolution order, using C3 linearization.
-
-        It returns the list of ancestors sorted by the mro.
-        This will raise `NotImplementedError` for old-style classes, since
-        they don't have the concept of MRO.
-        """
-        if not self.newstyle:
-            raise NotImplementedError(
-                "Could not obtain mro for old-style classes.")
-
+    def _compute_mro(self, context=None):
         inferred_bases = list(self._inferred_bases(context=context))
         bases_mro = []
         for base in inferred_bases:
@@ -1769,7 +1793,7 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG,
                 continue
 
             try:
-                mro = base.mro(context=context)
+                mro = base._compute_mro(context=context)
                 bases_mro.append(mro)
             except NotImplementedError:
                 # Some classes have in their ancestors both newstyle and
@@ -1784,6 +1808,20 @@ class ClassDef(mixins.FilterStmtsMixin, LocalsDictNodeNG,
         _verify_duplicates_mro(unmerged_mro, self, context)
         return _c3_merge(unmerged_mro, self, context)
 
+    def mro(self, context=None):
+        """Get the method resolution order, using C3 linearization.
+
+        It returns the list of ancestors sorted by the mro.
+        This will raise `NotImplementedError` for old-style classes, since
+        they don't have the concept of MRO.
+        """
+
+        if not self.newstyle:
+            raise NotImplementedError(
+                "Could not obtain mro for old-style classes.")
+
+        return self._compute_mro(context=context)
+
     def bool_value(self):
         return True
 
